# encoding: UTF-8
class String
  def to_multicash_h
    value = self
    h = {}
    value.split(/(?=\+)|(?=\-)/).each { |item|
      item.gsub!(/\s/,'')
      case item
      when /^([0-9\-+,\.]+)$/ # jen cislo
        quantity = $1
        commodity = $default_commodity
      when /^([0-9\-\+,\.]+)([a-zA-Z]+)$/ # cislo a komodita
        quantity = $1
        commodity = $2
      else
        quantity = '0'
        commodity = $default_commodity
      end
      ## sezere carky, pokud se zda ze oddeluji tisice - trochu heuristika
      # nejdrive necha jen nejpravejsi symbol z dvojice .,
      parts = quantity.rpartition(/,\./) 
      parts[0].gsub!(/,\./,'')
      quantity = parts.join
      # zbyla-li tam čárka
      if quantity.match(/,/)
        quantity.gsub!(/,(?=\d\d\d)/,'') # pak ji sežere pokud je před třema číslicema
        quantity.gsub!(/,/,'.') # a nebo promění na tečku
      end

      if quantity.to_f != 0 then h[commodity]=quantity.to_f end
    }
    if h.empty? then h = { $default_commodity => 0 } end
    return h
  end
  def currencify(options={})
    to_f.currencify(options)
  end
end

class Hash
  # aritmeticke operace s Hashema
  def +(hash2)
    case hash2 
      when Hash
        merge(hash2) { |key, old_value, new_value| 
          old_value.to_f + new_value.to_f 
        }.delete_if{ |key,value| value.to_f.near_zero? }
      when Float
        self+hash2.to_multicash_h
      when NilClass
        self
    end
  end

  def -(value)
    self + (-value.to_multicash_h)
  end

  def +@
    self
  end

  def -@
    self.collect_to_hash{ |value| -(value.to_f) }
  end

  # collect for Hashes http://oldrcrs.rubypal.com/rcr/show/278
  def collect_to_hash
    h = Hash.new
    keys.each do |key|
      h[key] = yield(self[key])
    end
    h
  end

  def to_multicash_h
    self
  end

  def to_s(options={})
    options={} if options.nil?
    default_options = 
      if options[:rich]
        {:with_html => true, :delimiter=>'&nbsp;', :force_zeros=>false}
      else
        {:delimiter=>',', :force_zeros=>false, :show_currency=>'always'}
      end
    options = default_options.merge(options) # default options

    show_currency = (options[:show_currency] == 'always') || (
      options[:show_currency] == 'nondefault' && (
        self.size>1 || (
          self.size == 1 && self.keys[0]!=$default_commodity
        )
      )
    )

    result = self.collect{|key, value|
      if (value == '') then value = 0 end
      if options[:with_html]
        value.to_f.currencify(:currency_symbol=>key, :delimiter=>options[:delimiter],
                              :html => true, :show_currency=>show_currency, 
                              :force_zeros=>options[:force_zeros], :round=>options[:round], :force_sign=>options[:force_sign])
      else
        # tento format se pouzije i pri vsech automatichkych konverzic multicash do stringu (napriklad pri stanoveni defaultni hodnoty pro editovani)
        value.to_f.currencify(:currency_symbol=>key, :delimiter => '', :show_currency=>show_currency, 
                              :force_zeros=>options[:force_zeros], :round=>options[:round], :force_sign=>options[:force_sign])
      end
    }.join('+').gsub(/^\+/,'').gsub(/\+\-/,'-')
    
    if options[:force_sign]
      result = result.gsub(/^(?!-)/,'+')
    end

    if options[:with_html]
      result = result.gsub(/(\+|\-)/,' \1&nbsp;')
    else
      result = result.gsub(/(\+|\-)/,' \1 ')
    end

    if result.empty? then result = '0' end
    result.html_safe
  end

  def negative?
    values.all?{ |value| value.to_f < 0.0001 } and not zero?
  end

  def positive?
    values.all?{ |value| value.to_f > -0.0001} and not zero?
  end

  def onlyCZK?
    keys == ['CZK']
  end

  def mixed?
    values.any?{ |value| value.to_f >= 0.0001} and
    values.any?{ |value| value.to_f <= -0.001}
  end

  def zero?
    values.all?{|value| value.to_f.abs<0.0001}
  end

  def czk
    self['CZK'].to_i rescue 0
  end
  
  def only_default_currecy?
    ((self.length == 0)) || ((self.length == 1) && (self.keys[0]==$default_commodity))
  end

  def equals?(h)
    (self-h).zero?
  end
  
  def reverse(yes_or_no)
    if yes_or_no
      {}-self
    else
      self
    end
  end
  
  def to_USD
    {'USD' => convert_to('USD')}
  end
  
  def convert_to(to_currency)
    @rates = { 'DKK' => 0.17362, 'CZK' => 0.04443, 'EUR'=>1.29231, 'USD'=>1}
    self.map { |from_currency,value|
      value*@rates[from_currency].to_f/@rates[to_currency].to_f
    }.sum || 0
  end
end

class Object
  def defaults_to(v)
    blank? ? v : self
  end
  def true  # polobastl ktery se pouzije v account/vysledovka pro account_type_test
    true
  end
end

class Float
  def currencify(options={})
    # :currency_before => false puts the currency symbol after the number
    # default format: 12,345,678.90 CZK
    options = ({:currency_symbol => $default_commodity, :delimiter => ",",
        :decimal_symbol => ".", :currency_before => false, :html => false,
        :show_currency => true, :force_zeros => false
    }).merge(options)

    # split integer and fractional parts 
    int, frac = ("%.2f" % self).split('.')
    # insert the delimiters
    int.gsub!(/(\d)(?=(\d\d\d)+(?!\d))/, "\\1#{options[:delimiter]}")

    if options[:html]
       options[:currency_symbol] = '<span class="curr">' + options[:currency_symbol] + '</span>'
    end
    if ((!(options[:force_zeros])) && (frac == '00')) || (options[:round])
      frac = ''
    else
      frac = options[:decimal_symbol] + frac
    end
    if options[:show_currency]
      if options[:currency_before]
        options[:currency_symbol] + int + frac
      else
        int + frac + (options[:html] ?  '&nbsp;': ' ') + options[:currency_symbol]
      end
    else
      int + frac
    end
  end
  def near_zero?
    self.abs < 0.0001
  end
end

class Fixnum
  def currencify(options={})
    self.to_f.currencify(options)
  end
  def to_multicash_h
    self.to_s.to_multicash_h
  end
end

class Float
  def to_multicash_h
    self.to_s.to_multicash_h
  end
end

class Integer
  def to_multicash_h
    self.to_s.to_multicash_h
  end
end

class NilClass
  def to_multicash_h
    {}
  end
end
